# Django library.
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


# Constants
# ------------------------------------------------------------------

__FIFTY_MEGABYTES = 50 * 1024 * 1024
__VALID_CONTENT_TYPES = 'image/jpeg', 'image/png', 'image/svg+xml'

# ------------------------------------------------------------------


def _from_settings(var, default):
    """
    Adds "MARXDOWX_" to the requested variable and retrieves its value
     from settings or returns the default.

    :param var: Variable to be retrieved.
    :type var: str
    :param default: Default value if the variable is not defined.
    :return: Value corresponding to 'var'.
    """
    return getattr(settings, 'MARKDOWNX_' + var, default)


# Markdownify
# ------------
MARKDOWNX_MARKDOWNIFY_FUNCTION = _from_settings('MARKDOWNIFY_FUNCTION', 'markdownx.utils.markdownify')


# Markdown extensions
# --------------------
MARKDOWNX_MARKDOWN_EXTENSIONS = _from_settings('MARKDOWN_EXTENSIONS', list())

MARKDOWNX_MARKDOWN_EXTENSION_CONFIGS = _from_settings('MARKDOWN_EXTENSION_CONFIGS', dict())


# Markdown urls
# --------------
MARKDOWNX_URLS_PATH = _from_settings('URLS_PATH', '/markdownx/markdownify/')

MARKDOWNX_UPLOAD_URLS_PATH = _from_settings('UPLOAD_URLS_PATH', '/markdownx/upload/')


# Media path
# -----------
MARKDOWNX_MEDIA_PATH = _from_settings('MEDIA_PATH', 'markdownx/')


# Image
# ------
MARKDOWNX_UPLOAD_MAX_SIZE = _from_settings('UPLOAD_MAX_SIZE', __FIFTY_MEGABYTES)

MARKDOWNX_UPLOAD_CONTENT_TYPES = _from_settings('UPLOAD_CONTENT_TYPES', __VALID_CONTENT_TYPES)

MARKDOWNX_IMAGE_MAX_SIZE = _from_settings('IMAGE_MAX_SIZE', dict(size=(500, 500), quality=90))


# Editor
# -------
MARKDOWNX_EDITOR_RESIZABLE = _from_settings('EDITOR_RESIZABLE', True)


# ------------------------------------------------
# Translations
# ------------------------------------------------
# This is not called using `_from_settings` as
# it does not need "_MARKDOWNX" prefix.
LANGUAGES = getattr(
    settings,
    'LANGUAGES',
    (
        ('en', _('English')),
        ('pl', _('Polish')),
    )
)



from django.db import models
from django.contrib import admin

from markdownx.widgets import AdminMarkdownxWidget
from markdownx.models import MarkdownxField

from .models import MyModel


class MyModelAdmin(admin.ModelAdmin):
    formfield_overrides = {
        MarkdownxField: {'widget': AdminMarkdownxWidget},
        models.TextField: {'widget': AdminMarkdownxWidget},
    }

admin.site.register(MyModel, MyModelAdmin)

##
## SHORTER OPTION:
##

# from django.contrib import admin

# from markdownx.admin import MarkdownxModelAdmin

# from .models import MyModel

# admin.site.register(MyModel, MarkdownxModelAdmin)



# Python internal library.
from os import path, SEEK_END
from uuid import uuid4
from collections import namedtuple

# Django library.
from django import forms
from django.utils.six import BytesIO
from django.core.files.storage import default_storage
from django.utils.translation import ugettext_lazy as _
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.template.defaultfilters import filesizeformat

# Internal.
from .utils import scale_and_crop
from .settings import (
    MARKDOWNX_IMAGE_MAX_SIZE,
    MARKDOWNX_MEDIA_PATH,
    MARKDOWNX_UPLOAD_CONTENT_TYPES,
    MARKDOWNX_UPLOAD_MAX_SIZE,
)


class ImageForm(forms.Form):
    image = forms.FileField()

    # Separately defined as it needs to be processed a text file rather than image.
    _SVG_TYPE = 'image/svg+xml'

    _error_templates = {
        # No file is available to upload.
        'not_uploaded':
            forms.ValidationError(_('No files have been uploaded.')),

        # The file is of a format not defined in "settings.py"
        # or if default, in "markdownx/settings.py".
        'unsupported_format':
            forms.ValidationError(_('File type is not supported.')),

        # The file is larger in size that the maximum allow in "settings.py" (or the default).
        'invalid_size':
            lambda current: forms.ValidationError(
                _('Please keep file size under {max}. Current file size {current}').format(
                    max=filesizeformat(MARKDOWNX_UPLOAD_MAX_SIZE),
                    current=filesizeformat(current)
                )
            )
    }

    def save(self, commit=True):
        """
        Saves the uploaded image in the designated location.

        If image type is not SVG, a byteIO of image content_type is created and
        subsequently save; otherwise, the SVG is saved in its existing `charset`
        as an `image/svg+xml`.

        The dimension of image files (excluding SVG) are set using `PIL`.

        :param commit: If `True`, the file is saved to the disk; otherwise, it is held in the memory.
        :type commit: bool
        :return: An instance of saved image if `commit is True`, else `namedtuple(path, image)`.
        :rtype: bool, namedtuple
        """
        image = self.files.get('image')
        content_type = image.content_type
        file_name = image.name
        image_extension = content_type.split('/')[-1].upper()
        image_size = getattr(image, '_size')

        if content_type.lower() != self._SVG_TYPE:
            # Processing the raster graphic image:
            image = self._process_raster(image, image_extension)
            image_size = image.tell()

        # Processed file (or the actual file in the case of SVG) is now
        # saved in the memory as a Django object.
        uploaded_image = InMemoryUploadedFile(
            file=image,
            field_name=None,
            name=file_name,
            content_type=content_type,
            size=image_size,
            charset=None
        )

        return self._save(uploaded_image, file_name, commit)

    def _save(self, image, file_name, commit):
        """
        Final saving process, called internally after the image had processed.
        """
        # Defining a universally unique name for the file
        # to be saved on the disk.
        unique_file_name = self.get_unique_file_name(file_name)
        full_path = path.join(MARKDOWNX_MEDIA_PATH, unique_file_name)

        if commit:
            default_storage.save(full_path, image)
            return default_storage.url(full_path)

        # If `commit is False`, return the path and in-memory image.
        image_data = namedtuple('image_data', ['path', 'image'])
        return image_data(path=full_path, image=image)

    @staticmethod
    def _process_raster(image, extension):
        """
        Processing of raster graphic image.
        """
        # File needs to be uploaded and saved temporarily in
        # the memory for additional processing using PIL.
        thumb_io = BytesIO()
        preped_image = scale_and_crop(image, **MARKDOWNX_IMAGE_MAX_SIZE)
        preped_image.save(thumb_io, extension)
        thumb_io.seek(0, SEEK_END)
        return thumb_io

    @staticmethod
    def get_unique_file_name(file_name):
        """
        Generates a universally unique ID using Python `UUID` and attaches the extension of file name to it.

        :param file_name: Name of the uploaded file, including the extension.
        :type file_name: str
        :return: Universally unique ID, ending with the extension extracted from `file_name`.
        :rtype: str
        """
        file_name = "{unique_name}.{extension}".format(
            unique_name=uuid4(),
            extension=path.splitext(file_name)[1][1:]  # [1] is the extension, [1:] discards the dot.
        )
        return file_name

    def clean(self):
        """
        Checks the upload against allowed extensions and maximum size.

        :return: Upload
        """
        upload = self.cleaned_data.get('image')

        # -----------------------------------------------
        # See comments in `self._error_templates` for
        # additional information on each error.
        # -----------------------------------------------
        if not upload:
            raise self._error_templates['not_uploaded']

        content_type = upload.content_type
        file_size = getattr(upload, '_size')

        if content_type not in MARKDOWNX_UPLOAD_CONTENT_TYPES:
            raise self._error_templates['unsupported_format']
        elif file_size > MARKDOWNX_UPLOAD_MAX_SIZE:
            raise self._error_templates['invalid_size'](file_size)

        return upload



from django.http import HttpResponse, JsonResponse
from django.utils.module_loading import import_string
from django.views.generic.edit import View, FormView

from .forms import ImageForm
from .settings import MARKDOWNX_MARKDOWNIFY_FUNCTION


class MarkdownifyView(View):

    def post(self, request, *args, **kwargs):
        markdownify = import_string(MARKDOWNX_MARKDOWNIFY_FUNCTION)
        return HttpResponse(markdownify(request.POST['content']))


class ImageUploadView(FormView):

    template_name = "dummy.html"
    form_class = ImageForm
    success_url = '/'

    def form_invalid(self, form):
        response = super(ImageUploadView, self).form_invalid(form)
        if self.request.is_ajax():
            return JsonResponse(form.errors, status=400)
        else:
            return response

    def form_valid(self, form):
        image_path = form.save()
        response = super(ImageUploadView, self).form_valid(form)

        if self.request.is_ajax():
            image_code = '![]({})'.format(image_path)
            return JsonResponse({'image_code': image_code})
        else:
            return response



from django import forms

from markdownx.fields import MarkdownxFormField
from markdownx.widgets import MarkdownxWidget

class MyForm(forms.Form):
    markdownx_form_field1 = MarkdownxFormField(widget=MarkdownxWidget(attrs={'class':'custom-class-markdownx_form_field1'}))
    markdownx_form_field2 = MarkdownxFormField(widget=MarkdownxWidget(attrs={'class':'custom-class-markdownx_form_field2'}))



from django import forms
from django.template.loader import get_template
from django.contrib.admin import widgets

from .settings import (
    MARKDOWNX_EDITOR_RESIZABLE,
    MARKDOWNX_URLS_PATH,
    MARKDOWNX_UPLOAD_URLS_PATH,
)


class MarkdownxWidget(forms.Textarea):

    def render(self, name, value, attrs=None):
        attrs = self.build_attrs(attrs, name=name)
        print(attrs)

        if 'class' in attrs:
            attrs['class'] += ' markdownx-editor'
        else:
            attrs.update({'class':'markdownx-editor'})

        attrs['data-markdownx-editor-resizable'] = MARKDOWNX_EDITOR_RESIZABLE
        attrs['data-markdownx-urls-path'] = MARKDOWNX_URLS_PATH
        attrs['data-markdownx-upload-urls-path'] = MARKDOWNX_UPLOAD_URLS_PATH

        widget = super(MarkdownxWidget, self).render(name, value, attrs)

        template = get_template('markdownx/widget.html')

        return template.render({
            'markdownx_editor': widget,
        })

    class Media:
        js = (
            'markdownx/js/markdownx.js',
        )


class AdminMarkdownxWidget(MarkdownxWidget, widgets.AdminTextareaWidget):

    class Media:
        css = {
            'all': ('markdownx/admin/css/markdownx.css',)
        }
        js = (
            'markdownx/js/markdownx.js',
        )



from setuptools import setup, find_packages

import os
if 'vagrant' in str(os.environ):
    del os.link

def get_requirements():
    return open('requirements.txt').read().splitlines()


setup(
    name='django-markdownx',
    version='1.7.2',
    packages=find_packages(),
    include_package_data=True,
    description='django-markdownx is a Markdown editor built for Django.',
    long_description='''https://github.com/adi-/django-markdownx/

Key features
------------

* raw editing
* live preview
* drag&drop image uploads (stored locally in `MEDIA` folder)
* customizable image insertion tag
* image filtering using content types and max file size
* image manipulations (compression, size, cropping, upscaling)
* pre-&post- text altering
* easy template customization for layout purposes
* multiple editors on one page
* Django Admin support

Preview
-------

.. image:: https://github.com/adi-/django-markdownx/raw/master/django-markdownx-preview.gif?raw=true
   :target: https://github.com/adi-/django-markdownx
   :alt: django-markdownx preview

*(using Bootstrap for layout and styling)*
''',
    url='https://github.com/adi-/django-markdownx',
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: JavaScript',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='django markdown markdownx django-markdownx editor image upload drag&drop',
    tests_require=get_requirements(),
    test_suite='runtests',
    install_requires=get_requirements(),

)



from django import forms

from .widgets import (
    MarkdownxWidget,
    AdminMarkdownxWidget,
)


class MarkdownxFormField(forms.CharField):

    def __init__(self, *args, **kwargs):
        super(MarkdownxFormField, self).__init__(*args, **kwargs)

        if issubclass(self.widget.__class__, forms.widgets.MultiWidget):
            if not any([
                issubclass(x.__class__, MarkdownxWidget)
                for x in self.widget.widgets
            ]):
                self.widget = MarkdownxWidget()
        elif not issubclass(self.widget.__class__, MarkdownxWidget):
            self.widget = MarkdownxWidget()
